'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _options = require('../options');

var _options2 = _interopRequireDefault(_options);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var _utilities = require('./utilities');

var _chunks = require('../chunks');

var _chunks2 = _interopRequireDefault(_chunks);

var USAGE = 'Usage: chunkify.each(Array array, Function fn, [Object options])';

var each = function each(array, fn) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  (0, _utilities.ok_usage)(array, fn, USAGE);

  var _ChunkifyOptions$of = _options2['default'].of(options);

  var chunk = _ChunkifyOptions$of.chunk;
  var delay = _ChunkifyOptions$of.delay;
  var scope = _ChunkifyOptions$of.scope;

  var iterator = _chunks2['default'].of({ chunk: chunk, range: array.length });
  var resume = function resume(resolve, reject) {
    var next = iterator.next();
    if (next.done) {
      return resolve();
    }
    var _next$value = next.value;
    var index = _next$value.index;
    var pause = _next$value.pause;

    var item = array[index];
    try {
      fn.call(scope, item, index);
    } catch (error) {
      return reject({ error: error, item: item, index: index });
    }
    if (pause) {
      return setTimeout(resume, delay, resolve, reject);
    }
    resume(resolve, reject);
  };
  return new Promise(resume);
};

exports['default'] = each;
module.exports = exports['default'];