'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _chunks = require('./chunks');

var _chunks2 = _interopRequireDefault(_chunks);

var _options = require('./options');

var _options2 = _interopRequireDefault(_options);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var USAGE = 'Usage: chunkify.loop(Number range, Function fn, [Object options])';

var ok_usage = function ok_usage(range, fn) {
  if (!_underscore2['default'].isNumber(range)) {
    throw new Error(USAGE + ' - bad range');
  } else if (!_underscore2['default'].isFunction(fn)) {
    throw new Error(USAGE + ' - bad fn');
  }
};

var loop = function loop(range, fn) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  ok_usage(range, fn);

  var _ChunkifyOptions$of = _options2['default'].of(options);

  var scope = _ChunkifyOptions$of.scope;
  var chunk = _ChunkifyOptions$of.chunk;
  var delay = _ChunkifyOptions$of.delay;

  var iterator = _chunks2['default'].of({ chunk: chunk, range: range });
  var resume = function resume(resolve, reject) {
    var next = iterator.next();
    if (next.done) {
      return resolve();
    }
    var _next$value = next.value;
    var index = _next$value.index;
    var pause = _next$value.pause;

    try {
      fn.call(scope, index);
    } catch (error) {
      return reject({ error: error, index: index });
    }
    if (pause) {
      return setTimeout(resume, delay, resolve, reject);
    }
    resume(resolve, reject);
  };
  return new Promise(resume);
};

exports['default'] = loop;
module.exports = exports['default'];