'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _chunkify = require('../chunkify');

var _chunkify2 = _interopRequireDefault(_chunkify);

var _options = require('../options');

var _options2 = _interopRequireDefault(_options);

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

var USAGE = 'Usage: chunkify.range(Function fn, Number final, [Object options])';

var ok_usage = function ok_usage(fn, final, options) {
  if (!_underscore2['default'].isFunction(fn)) {
    throw new Error(USAGE + ' - bad fn; not a function');
  } else if (!_underscore2['default'].isNumber(final)) {
    throw new Error(USAGE + ' - bad final; not a number');
  }
  var start = options.start;

  if (start != null) {
    if (!_underscore2['default'].isNumber(start)) {
      throw new Error(USAGE + ' - bad start; not a number');
    } else if (start > final) {
      throw new Error(USAGE + ' - bad start; it\'s greater than final');
    }
  }
};

var range = function range(fn, final) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  ok_usage(fn, final, options);

  var _ChunkifyOptions$of = _options2['default'].of(options);

  var scope = _ChunkifyOptions$of.scope;
  var chunk = _ChunkifyOptions$of.chunk;
  var delay = _ChunkifyOptions$of.delay;

  var iterator = _chunkify2['default'].range({
    start: options.start || 0,
    final: final,
    chunk: chunk
  });
  var resume = function resume(resolve, reject) {
    var next = iterator.next();
    if (next.done) {
      return resolve();
    }
    var _next$value = next.value;
    var index = _next$value.index;
    var pause = _next$value.pause;

    try {
      fn.call(scope, index);
    } catch (error) {
      return reject({ error: error, index: index });
    }
    if (pause) {
      setTimeout(resume, delay, resolve, reject);
    } else {
      resume(resolve, reject);
    }
  };
  return new Promise(resume);
};

exports['default'] = range;
module.exports = exports['default'];